
#include "compute_BLER_CR.h"

/*===================================================================
FUNCTION: ComputeBLER()

AUTHOR: SJ, KR, DW

DESCRIPTION: BLER from the Link-Level Result

NOTES:
===================================================================*/
double ComputeBLER(LL_CURVE_LOOK  input)
{
	if (input.effective_code_rate < 1)
	{
		bler_computing_setting(input);
		bler_computing_target_setting();
		bler_curve_generation();
		bler_computation();
	}
	else
	{

		cerr << "Something Wrong Code rate" << endl;

		//bler = 1;
	}

	return (bler);
}



/*===================================================================
FUNCTION: bler_computing_setting()

AUTHOR: KR

DESCRIPTION:

NOTES:
===================================================================*/
void bler_computing_setting(LL_CURVE_LOOK  input)
{

	bler = 0.0;
	modulation_type = input.modulation_idx;
	cr = input.effective_code_rate;


	x_axis = linear2dB(input.snr);


	if (modulation_type == QPSK)   { modulation_index = 0; }
	else if (modulation_type == QAM16)   { modulation_index = 5; }
	else if (modulation_type == QAM64)   { modulation_index = 10; }

}

/*===================================================================
FUNCTION: bler_computing_target_setting()

AUTHOR: KR

DESCRIPTION:

NOTES:
===================================================================*/
void bler_computing_target_setting()
{
	if (modulation_type == QPSK)
	for (int target_idx = 0; target_idx < 21; target_idx++)
	{
		if (cr >= CR_curves[target_idx] && cr < CR_curves[target_idx + 1])
		{
			cr_target = target_idx;
		}
	}

	if (modulation_type == QAM16)
	for (int target_idx = 21; target_idx < 30; target_idx++)
	{
		if (cr >= CR_curves[target_idx] && cr < CR_curves[target_idx + 1])
		{
			cr_target = target_idx;
		}
	}

	if (modulation_type == QAM64)
	for (int target_idx = 30; target_idx < 38; target_idx++)
	{
		if (cr >= CR_curves[target_idx] && cr < CR_curves[target_idx + 1])
		{
			cr_target = target_idx;
		}
	}
}

/*===================================================================
FUNCTION: bler_curve_generation()

AUTHOR: KR

DESCRIPTION:

NOTES:
===================================================================*/
void bler_curve_generation()
{
	for (int i = 0; i < 7; i++)
	{
		cr_bler_curve[i][0] = linear_cr(CR_curves[cr_target], NEW_TABLE[cr_target][i][0],
			CR_curves[cr_target + 1], NEW_TABLE[cr_target + 1][i][0], cr);
		cr_bler_curve[i][1] = NEW_TABLE[0][i][1];
	}
}

/*===================================================================
FUNCTION: bler_computation()

AUTHOR: KR

DESCRIPTION:

NOTES:
===================================================================*/
void bler_computation()
{
	if (x_axis < cr_bler_curve[0][0])
	{
		bler = 1;
	}
	for (int i = 0; i < 6; i++)
	{
		if (x_axis >= cr_bler_curve[i][0] && x_axis < cr_bler_curve[i + 1][0])
		{
			bler = linear_approx_yais(cr_bler_curve[i][0], log10(cr_bler_curve[i][1]),
				cr_bler_curve[i + 1][0], log10(cr_bler_curve[i + 1][1]), x_axis);
			bler = pow(10., bler);
		}
	}
	if (x_axis >= cr_bler_curve[6][0])
	{
		bler = 0;
	}
}


